class TextTag:
    """This class provides a set of SpaCy-style constants to use with Tag(kind="token")."""

    TextWithWs = "textWithWs"
    Whitespace = "whitespace"
    Head = "head"
    LeftEdge = "leftEdge"
    RightEdge = "rightEdge"
    EntType = "entType"
    EntIob = "entIob"
    Lemma = "lemma"
    Normalized = "normalized"
    Shape = "shape"
    Prefix = "prefix"
    Suffix = "suffix"
    IsAlpha = "isAlpha"
    IsAscii = "isAscii"
    IsDigit = "isDigit"
    IsTitle = "isTitle"
    IsPunct = "isPunct"
    IsLeftPunct = "isLeftPunct"
    IsRightPunct = "isRightPunct"
    IsSpace = "isSpace"
    IsBracket = "isBracket"
    IsQuote = "isQuote"
    IsCurrency = "isCurrency"
    LikeUrl = "likeUrl"
    LikeNum = "likeNum"
    LikeEmail = "likeEmail"
    IsOov = "isOov"
    IsStop = "isStop"
    Lang = "lang"
    Embedding = "embedding"
